@@ -30,19 +30,20 @@ class Recipe(GenericBaseRecipe):
 
   def install(self):
 
-    parameters = [
+    args = [
+      self.options['equeue-binary'],
       '--database', self.options['database'],
-      '-l', self.options['log'],
+      '--logfile', self.options['log'],
+      '--lockfile', self.options['lockfile']
     ]
 
+    if 'takeover-triggered-file-path' in self.options:
+      args += ('--takeover-triggered-file-path',
+               self.options['takeover-triggered-file-path'])
+
     if 'loglevel' in self.options:
-      parameters.extend(['--loglevel', self.options['loglevel']])
+      args += '--loglevel', self.options['loglevel']
 
-    parameters.append(self.options['socket'])
-
-    wrapper = self.createWrapper(name=self.options['wrapper'],
-                                 command=self.options['equeue-binary'],
-                                 parameters=parameters)
-
-    return [wrapper]
+    args.append(self.options['socket'])
 
+    return self.createWrapper(self.options['wrapper'], args)
