@@ -24,8 +24,36 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #
 ##############################################################################
-from slapos.recipe.librecipe import GenericBaseRecipe
+import os
+import shlex
 from zc.buildout import UserError
+from .librecipe import GenericBaseRecipe
+
+class Cluster(object):
+
+  def __init__(self, buildout, name, options):
+    masters = options.setdefault('masters', '')
+    result_dict = {
+      'connection-admin': [],
+      'connection-master': [],
+    }
+    node_list = []
+    for node in sorted(options['nodes'].split()):
+      node = buildout[node]
+      node_list.append(node)
+      for k, v in result_dict.iteritems():
+        x = node[k]
+        if x:
+          v.append(x)
+    options['admins'] = ' '.join(result_dict.pop('connection-admin'))
+    x = ' '.join(result_dict.pop('connection-master'))
+    if masters != x:
+      options['masters'] = x
+      for node in node_list:
+        node['config-masters'] = x
+        node.recipe.__init__(buildout, node.name, node)
+
+  install = update = lambda self: None
 
 class NeoBaseRecipe(GenericBaseRecipe):
 
@@ -41,8 +69,9 @@ class NeoBaseRecipe(GenericBaseRecipe):
       # Only then can this recipe start succeeding and actually doing anything
       # useful, as per NEO deploying constraints.
       raise UserError('"masters" parameter is mandatory')
-    option_list = [
+    args = [
       options['binary'],
+      # Keep the -l option first, as expected by logrotate snippets.
       '-l', options['logfile'],
       '-m', options['masters'],
       '-b', self._getBindingAddress(),
@@ -50,12 +79,16 @@ class NeoBaseRecipe(GenericBaseRecipe):
       #'-n', options['name'],
       '-c', options['cluster'],
     ]
-    option_list.extend(self._getOptionList())
-    return [self.createPythonScript(
-      options['wrapper'],
-      'slapos.recipe.librecipe.execute.execute',
-      option_list
-    )]
+    if options['ssl']:
+      etc = os.path.join(self.buildout['buildout']['directory'], 'etc', '')
+      args += (
+        '--ca', etc + 'ca.crt',
+        '--cert', etc + 'neo.crt',
+        '--key', etc + 'neo.key',
+        )
+    args += self._getOptionList()
+    args += shlex.split(options.get('extra-options', ''))
+    return self.createWrapper(options['wrapper'], args)
 
   def _getBindingAddress(self):
     options = self.options
@@ -76,11 +109,19 @@ class Storage(NeoBaseRecipe):
   _binding_port_mandatory = False
 
   def _getOptionList(self):
-    return [
+    r = [
       '-d', self.options['database-parameters'],
       '-a', self.options['database-adapter'],
       '-w', self.options['wait-database'],
     ]
+    engine = self.options.get('engine')
+    if engine: # old versions of NEO don't support -e
+      r  += '-e', engine
+    if self.options.get('dedup'):
+      r.append('--dedup')
+    if self.options.get('disable-drop-partitions'):
+      r.append('--disable-drop-partitions')
+    return r
 
 class Admin(NeoBaseRecipe):
   def _getOptionList(self):
@@ -89,7 +130,13 @@ class Admin(NeoBaseRecipe):
 class Master(NeoBaseRecipe):
   def _getOptionList(self):
     options = self.options
-    return [
+    r = [
       '-p', options['partitions'],
       '-r', options['replicas'],
+      '-A', options['autostart'],
     ]
+    for x in (('-C', options['upstream-cluster']),
+              ('-M', options['upstream-masters'])):
+      if x[1]:
+        r += x
+    return r
