@@ -25,10 +25,11 @@
 #
 ##############################################################################
 
-import md5
+import hashlib
 import os
 import subprocess
 import textwrap
+import shutil
 from zc.buildout import UserError
 
 from slapos.recipe.librecipe import GenericBaseRecipe
@@ -79,11 +80,19 @@ class Recipe(GenericBaseRecipe):
         # if the pgdata already exists, skip all steps, we don't need to do anything.
 
         if not os.path.exists(pgdata):
-            self.createCluster()
-            self.createConfig()
-            self.createDatabase()
-            self.updateSuperuser()
-            self.createRunScript()
+            try:
+                self.createCluster()
+                self.createConfig()
+                self.createDatabase()
+                self.updateSuperuser()
+                self.createRunScript()
+            except:
+                # do not leave half-installed postgresql - else next time we
+                # run we won't update it.
+                shutil.rmtree(pgdata)
+                raise
+
+
 
         # install() methods usually return the pathnames of managed files.
         # If they are missing, they will be rebuilt.
@@ -140,7 +149,7 @@ class Recipe(GenericBaseRecipe):
                     lc_time = 'en_US.UTF-8'
                     default_text_search_config = 'pg_catalog.english'
 
-                    unix_socket_directory = '%s'
+                    unix_socket_directories = '%s'
                     unix_socket_permissions = 0700
                     """ % (
                         ','.join(ipv4.union(ipv6)),
@@ -186,7 +195,7 @@ class Recipe(GenericBaseRecipe):
         password = self.options['password']
 
         # encrypt the password to avoid storing in the logs
-        enc_password = 'md5' + md5.md5(password+user).hexdigest()
+        enc_password = 'md5' + hashlib.md5(password+user).hexdigest()
 
         self.runPostgresCommand(cmd="""ALTER USER "%s" ENCRYPTED PASSWORD '%s'""" % (user, enc_password))
 
